@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2013 IBM Corporation and others.
+ * Copyright (c) 2005, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,6 +14,8 @@
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.dom.parser.cpp;
 
+import static org.eclipse.cdt.internal.core.dom.parser.cpp.CPPClassScope.createInheritedConsructors;
+
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.dom.IName;
 import org.eclipse.cdt.core.dom.ast.DOMException;
@@ -26,11 +28,13 @@ import org.eclipse.cdt.core.dom.ast.IScope;
 import org.eclipse.cdt.core.dom.ast.IType;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPBase;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPClassSpecialization;
+import org.eclipse.cdt.core.dom.ast.cpp.ICPPClassTemplatePartialSpecialization;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPClassType;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPConstructor;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPField;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPMethod;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPNamespace;
+import org.eclipse.cdt.core.dom.ast.cpp.ICPPParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPParameterPackType;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateParameter;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateParameterMap;
@@ -46,8 +50,16 @@ import org.eclipse.cdt.internal.core.dom.parser.cpp.semantics.SemanticUtil;
  * For safe usage in index bindings, all fields need to be final or volatile.
  */
 public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializationScope {
-	final private ICPPClassSpecialization specialClass;
-	private volatile ICPPBase[] fBases; // Used by the pdom bindings, needs to be volatile.
+	private final ICPPClassSpecialization specialClass;
+	// The following fields are used by the PDOM bindings and need to be volatile.
+	private volatile ICPPBase[] fBases;
+	private volatile ICPPMethod[] ownInheritedConstructors;
+	private final ThreadLocal<Boolean> fComputingBases = new ThreadLocal<Boolean>() {
+		@Override
+		protected Boolean initialValue() {
+			return false;
+		}
+	};
 
 	public AbstractCPPClassSpecializationScope(ICPPClassSpecialization specialization) {
 		this.specialClass= specialization;
@@ -79,16 +91,17 @@ public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializat
 
 		ICPPClassType specialized = specialClass.getSpecializedBinding();
 		IScope classScope = specialized.getCompositeScope();
-		IBinding[] bindings = classScope != null ? classScope.getBindings(new ScopeLookupData(name, resolve, false)) : null;
+		IBinding[] bindings = classScope != null ?
+				classScope.getBindings(new ScopeLookupData(name, resolve, false)) : null;
 		
 		if (bindings == null)
 			return null;
     	
-		IBinding[] specs = new IBinding[0];
+		IBinding[] specs = IBinding.EMPTY_BINDING_ARRAY;
 		for (IBinding binding : bindings) {
-			specs = ArrayUtil.append(IBinding.class, specs, specialClass.specializeMember(binding, name));
+			specs = ArrayUtil.append(specs, specialClass.specializeMember(binding, name));
 		}
-		specs = ArrayUtil.trim(IBinding.class, specs);
+		specs = ArrayUtil.trim(specs);
     	return CPPSemantics.resolveAmbiguities(name, specs);
 	}
 
@@ -107,17 +120,28 @@ public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializat
 			return IBinding.EMPTY_BINDING_ARRAY;
 
 	    IBinding[] bindings= classScope.getBindings(lookup);
-		IBinding[] result= null;
+		IBinding[] result= IBinding.EMPTY_BINDING_ARRAY;
+		int n = 0;
 		for (IBinding binding : bindings) {
 			if (binding == specialized ||
-					(binding instanceof ICPPClassType && specialized.isSameType((IType) binding))) {
+					(binding instanceof ICPPClassType && areSameTypesModuloPartialSpecialization(specialized, (IType) binding))) {
 				binding= specialClass;
 			} else {
 				binding= specialClass.specializeMember(binding, lookup.getLookupPoint());
 			}
-			result = ArrayUtil.append(IBinding.class, result, binding);
+			result = ArrayUtil.appendAt(result, n++, binding);
 		}
-		return ArrayUtil.trim(IBinding.class, result);
+		return ArrayUtil.trim(result, n);
+	}
+
+	private static boolean areSameTypesModuloPartialSpecialization(IType type1, IType type2) {
+		while (type1 instanceof ICPPClassTemplatePartialSpecialization) {
+			type1 = ((ICPPClassTemplatePartialSpecialization) type1).getPrimaryClassTemplate();
+		}
+		while (type2 instanceof ICPPClassTemplatePartialSpecialization) {
+			type2 = ((ICPPClassTemplatePartialSpecialization) type2).getPrimaryClassTemplate();
+		}
+		return type1.isSameType(type2);
 	}
 	
 	@Override
@@ -128,48 +152,56 @@ public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializat
 	@Override
 	public ICPPBase[] getBases(IASTNode point) {
 		if (fBases == null) {
-			ICPPBase[] result = null;
-			ICPPBase[] bases = ClassTypeHelper.getBases(specialClass.getSpecializedBinding(), point);
-			if (bases.length == 0) {
-				fBases= bases;
-			} else {
-				final ICPPTemplateParameterMap tpmap = specialClass.getTemplateParameterMap();
-				for (ICPPBase base : bases) {
-					IBinding origClass = base.getBaseClass();
-					if (origClass instanceof ICPPTemplateParameter && ((ICPPTemplateParameter) origClass).isParameterPack()) {
-						IType[] specClasses= CPPTemplates.instantiateTypes(new IType[] { new CPPParameterPackType((IType) origClass) },
-								tpmap, -1, specialClass, point);
-						if (specClasses.length == 1 && specClasses[0] instanceof ICPPParameterPackType) {
-							result= ArrayUtil.append(ICPPBase.class, result, base);
-						} else {
-							for (IType specClass : specClasses) {
-								ICPPBase specBase = base.clone();
-								specClass = SemanticUtil.getUltimateType(specClass, false);
-								if (specClass instanceof IBinding && !(specClass instanceof IProblemBinding)) {
-									specBase.setBaseClass((IBinding) specClass);
-									result = ArrayUtil.append(ICPPBase.class, result, specBase);
+			if (fComputingBases.get()) {
+				return ICPPBase.EMPTY_BASE_ARRAY;  // avoid recursion
+			}
+			fComputingBases.set(true);
+			try {
+				ICPPBase[] result = ICPPBase.EMPTY_BASE_ARRAY;
+				ICPPBase[] bases = ClassTypeHelper.getBases(specialClass.getSpecializedBinding(), point);
+				if (bases.length == 0) {
+					fBases= bases;
+				} else {
+					final ICPPTemplateParameterMap tpmap = specialClass.getTemplateParameterMap();
+					for (ICPPBase base : bases) {
+						IBinding origClass = base.getBaseClass();
+						if (origClass instanceof ICPPTemplateParameter && ((ICPPTemplateParameter) origClass).isParameterPack()) {
+							IType[] specClasses= CPPTemplates.instantiateTypes(new IType[] { new CPPParameterPackType((IType) origClass) },
+									tpmap, -1, specialClass, point);
+							if (specClasses.length == 1 && specClasses[0] instanceof ICPPParameterPackType) {
+								result= ArrayUtil.append(result, base);
+							} else {
+								for (IType specClass : specClasses) {
+									ICPPBase specBase = base.clone();
+									specClass = SemanticUtil.getUltimateType(specClass, false);
+									if (specClass instanceof IBinding && !(specClass instanceof IProblemBinding)) {
+										specBase.setBaseClass((IBinding) specClass);
+										result = ArrayUtil.append(result, specBase);
+									}
 								}
 							}
+							continue;
 						}
-						continue;
-					}
-					if (origClass instanceof IType) {
-						ICPPBase specBase = base.clone();
-						ICPPClassSpecialization specializationContext = specialClass;
-						if (specialClass.getOwner() instanceof ICPPClassSpecialization) {
-							specializationContext = (ICPPClassSpecialization) specialClass.getOwner();
+						if (origClass instanceof IType) {
+							ICPPBase specBase = base.clone();
+							ICPPClassSpecialization specializationContext = specialClass;
+							if (specialClass.getOwner() instanceof ICPPClassSpecialization) {
+								specializationContext = (ICPPClassSpecialization) specialClass.getOwner();
+							}
+							IType specClass= CPPTemplates.instantiateType((IType) origClass, tpmap, -1, specializationContext, point);
+							specClass = SemanticUtil.getUltimateType(specClass, false);
+							if (specClass instanceof IBinding && !(specClass instanceof IProblemBinding)) {
+								specBase.setBaseClass((IBinding) specClass);
+							}
+							result = ArrayUtil.append(result, specBase);
 						}
-						IType specClass= CPPTemplates.instantiateType((IType) origClass, tpmap, -1, specializationContext, point);
-						specClass = SemanticUtil.getUltimateType(specClass, false);
-						if (specClass instanceof IBinding && !(specClass instanceof IProblemBinding)) {
-							specBase.setBaseClass((IBinding) specClass);
-						}
-						result = ArrayUtil.append(ICPPBase.class, result, specBase);
 					}
+					result= ArrayUtil.trim(result);
+					fBases= result;
+					return result;
 				}
-				result= ArrayUtil.trim(ICPPBase.class, result);
-				fBases= result;
-				return result;
+			} finally {
+				fComputingBases.set(false);
 			}
 		}
 		return fBases;
@@ -204,7 +236,10 @@ public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializat
 	public ICPPMethod[] getImplicitMethods(IASTNode point) {
 		ICPPClassType origClass = specialClass.getSpecializedBinding();
 		ICPPMethod[] methods= ClassTypeHelper.getImplicitMethods(origClass, point);
-		return specializeMembers(methods, point);
+		ICPPMethod[] specializedMembers = specializeMembers(methods, point);
+		// Add inherited constructors.
+		ICPPMethod[] inheritedConstructors = getOwnInheritedConstructors(point);
+		return ArrayUtil.addAll(specializedMembers, inheritedConstructors);
 	}
 
 	@Override
@@ -223,7 +258,52 @@ public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializat
 	@Override
 	public ICPPConstructor[] getConstructors(IASTNode point) {
 		ICPPConstructor[] ctors= ClassTypeHelper.getConstructors(specialClass.getSpecializedBinding(), point);
-		return specializeMembers(ctors, point);
+		ICPPConstructor[] specializedCtors = specializeMembers(ctors, point);
+		// Add inherited constructors.
+		ICPPMethod[] inheritedConstructors = getOwnInheritedConstructors(specializedCtors, point);
+		return ArrayUtil.addAll(specializedCtors, inheritedConstructors);
+	}
+
+	/**
+	 * Returns the inherited constructors that are not specializations of the inherited constructors
+	 * of the specialized class.
+	 */
+	private ICPPMethod[] getOwnInheritedConstructors(ICPPConstructor[] existingConstructors,
+			IASTNode point) {
+		if (ownInheritedConstructors == null) {
+	        if (!hasInheritedConstructorsSources(point))
+	        	return ICPPMethod.EMPTY_CPPMETHOD_ARRAY;
+	
+	        IType[][] existingConstructorParamTypes = new IType[existingConstructors.length][];
+			for (int i = 0; i < existingConstructors.length; i++) {
+				ICPPParameter[] params = existingConstructors[i].getParameters();
+				IType[] types = new IType[params.length];
+				for (int j = 0; j < params.length; j++) {
+					types[j] = params[j].getType();
+				}
+				existingConstructorParamTypes[i] = types;
+			}
+			ICPPMethod[] constructors = createInheritedConsructors(this, specialClass.getNameCharArray(),
+					getBases(point), existingConstructorParamTypes, point);
+			ownInheritedConstructors = constructors;
+		}
+		return ownInheritedConstructors;
+	}
+
+	private ICPPMethod[] getOwnInheritedConstructors(IASTNode point) {
+		if (ownInheritedConstructors != null)
+			return ownInheritedConstructors;
+		ICPPConstructor[] ctors= ClassTypeHelper.getConstructors(specialClass.getSpecializedBinding(), point);
+		ICPPConstructor[] specializedCtors = specializeMembers(ctors, point);
+		return getOwnInheritedConstructors(specializedCtors, point);
+	}
+
+	private boolean hasInheritedConstructorsSources(IASTNode point) {
+		for (ICPPBase base : getBases(point)) {
+			if (base.isInheritedConstructorsSource())
+				return true;
+		}
+		return false;
 	}
 
 	@Override
@@ -262,14 +342,14 @@ public class AbstractCPPClassSpecializationScope implements ICPPClassSpecializat
 	}
 
 	@Override
-	public String toString() {
-		IName name = getScopeName();
-		return name != null ? name.toString() : String.valueOf(specialClass);
+	public EScopeKind getKind() {
+		return EScopeKind.eClassType;
 	}
 
 	@Override
-	public EScopeKind getKind() {
-		return EScopeKind.eClassType;
+	public String toString() {
+		IName name = getScopeName();
+		return name != null ? name.toString() : String.valueOf(specialClass);
 	}
 
 	// Note: equals() and hashCode() are overridden because multiple instances
