@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2012 Institute for Software, HSR Hochschule fuer Technik  
+ * Copyright (c) 2008, 2014 Institute for Software, HSR Hochschule fuer Technik
  * Rapperswil, University of applied sciences and others
  * All rights reserved. This program and the accompanying materials 
  * are made available under the terms of the Eclipse Public License v1.0 
@@ -13,6 +13,9 @@
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.dom.rewrite.astwriter;
 
+import java.util.EnumSet;
+
+import org.eclipse.cdt.core.dom.ast.IASTAttributeOwner;
 import org.eclipse.cdt.core.dom.ast.IASTCompositeTypeSpecifier;
 import org.eclipse.cdt.core.dom.ast.IASTDeclSpecifier;
 import org.eclipse.cdt.core.dom.ast.IASTDeclaration;
@@ -153,6 +156,9 @@ public class DeclSpecWriter extends NodeWriter {
 
 	private void writeElaboratedTypeSec(IASTElaboratedTypeSpecifier elabType) {
 		scribe.printStringSpace(getElabTypeString(elabType.getKind()));
+		if (elabType instanceof IASTAttributeOwner) {
+			writeAttributes((IASTAttributeOwner) elabType, EnumSet.of(SpaceLocation.AFTER));
+		}
 		elabType.getName().accept(visitor);
 	}
 
@@ -207,6 +213,9 @@ public class DeclSpecWriter extends NodeWriter {
 
 	private void writeEnumSpec(IASTEnumerationSpecifier enumSpec) {
 		scribe.printStringSpace(Keywords.ENUM);
+		if (enumSpec instanceof IASTAttributeOwner) {
+			writeAttributes((IASTAttributeOwner) enumSpec, EnumSet.of(SpaceLocation.AFTER));
+		}
 		enumSpec.getName().accept(visitor);
 		scribe.print('{');
 		scribe.printSpace();
@@ -233,6 +242,9 @@ public class DeclSpecWriter extends NodeWriter {
 	private void writeCompositeTypeSpecifier(IASTCompositeTypeSpecifier compDeclSpec) {
 		boolean hasTrailingComments = hasTrailingComments(compDeclSpec.getName());
 		scribe.printStringSpace(getCPPCompositeTypeString(compDeclSpec.getKey()));
+		if (compDeclSpec instanceof IASTAttributeOwner) {
+			writeAttributes((IASTAttributeOwner) compDeclSpec, EnumSet.of(SpaceLocation.AFTER));
+		}
 		compDeclSpec.getName().accept(visitor);
 		if (compDeclSpec instanceof ICPPASTCompositeTypeSpecifier) {
 			ICPPASTCompositeTypeSpecifier cppComp = (ICPPASTCompositeTypeSpecifier) compDeclSpec;
@@ -356,6 +368,7 @@ public class DeclSpecWriter extends NodeWriter {
 	private void writeCPPSimpleDeclSpec(ICPPASTSimpleDeclSpecifier simpDeclSpec) {
 		printQualifiers(simpDeclSpec);
 		scribe.print(getCPPSimpleDecSpecifier(simpDeclSpec));
+		writeAttributes(simpDeclSpec, EnumSet.of(SpaceLocation.BEFORE));
 		if (simpDeclSpec.getType() == IASTSimpleDeclSpecifier.t_typeof) {
 			scribe.printSpace();
 			visitNodeIfNotNull(simpDeclSpec.getDeclTypeExpression());
@@ -377,7 +390,7 @@ public class DeclSpecWriter extends NodeWriter {
 			scribe.printStringSpace(Keywords.SHORT);
 		} else if (simpDeclSpec.isLong()) {
 			scribe.printStringSpace(Keywords.LONG);
-		} else if (simpDeclSpec.isLongLong()) {			
+		} else if (simpDeclSpec.isLongLong()) {
 			scribe.printStringSpace(Keywords.LONG);
 			scribe.printStringSpace(Keywords.LONG);
 		}
